/*
* This file is part of the Tecodino-Sensor-Project.
* Even if you can use Tecodino with GR-16 and GR-12
* the software has nothing to do with Graupner(R)!
*
* Copyright(c) Stefan Radolf, 2011
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/


//#include <Wire.h>
#include "GMessage.h"


uint8_t message_array[36] = {
  0x00,0x7c,0x8d,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00,0x7d,0x00};


GMessage::GMessage(){
  Serial.println("VERSION_STRING");
}

/* 
 send_GMessage() dient zum Abschließenden 
 Senden der Telemetrie-Daten, wenn vorher für die 
 einzelnen Bytes sinnvolle Werte gesetzt wurden.
 */
void GMessage::send(){  
  message_array[MESSAGE_CHECKBYTE_POS] = checksumme(message_array);
  for(int i = 0; i < LENGTH; i++){
    Serial.print(message_array[i], BYTE);
    delay(PROZESSOR_GESCHWINDIGKEITS_TYP);
  }
}


/*
 akkustischer Alarm bei Unterspannung.
 ist private und muss nicht direkt verwendet werden.
*/
void GMessage::accoustic_ALARM_OFF(){
  message_array[MESSAGE_ALARMBYTE_POS] = ALARM_OFF;
}
void GMessage::accoustic_ALARM_LOW_VOLTAGE(){
  message_array[MESSAGE_ALARMBYTE_POS] = ALARM_LOW_VOLTAGE;
}

/* 
 Die 'checksumme()-Funktion' braucht vom Programmierer/Benutzer 
 nicht direkt aufgerufen werden. Sie ist _private und wird automatisch 
 beim Aufruf von send_GMessage() ausgeführt. Als Parameter wird das ganze 
 Bytearray verwendet; siehe Testdaten (ganz oben im Quelltext) 
 */
uint8_t GMessage::checksumme(uint8_t *sensordaten)
{
  uint8_t sum=  0x00;
  uint8_t checksumme = 0x00;
  uint8_t ganze_zahl_vor_dem_komma = 0;
  for (uint8_t index = MESSAGE_STARTBYTE_POS; index <= MESSAGE_ENDBYTE_POS; index++)
  {
    sum = sum + sensordaten[index];                 
  } 
  ganze_zahl_vor_dem_komma = (int8_t) sum / 0xff;
  checksumme = sum%0xff - 122 + ganze_zahl_vor_dem_komma;    
  return checksumme;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/* 
 Hauptspannung: Werte +0.1 bis +53.5 
 */
void GMessage::setMainVoltage(float amp){
  uint16_t val = (uint16_t) (amp * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  message_array[28] = low; 
  message_array[29] = high;    
}
/* 
 Akku Nummer 1 = Spannungssensor 1, Werte +0.1 bis +53.5 
 */
void GMessage::setVoltage1(float volt){
  uint16_t val = (uint16_t) (volt * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  message_array[10] = low; 
  message_array[11] = high;    
}
/* 
 Akku Nummer 2 = Spannungssensor 2, Werte +0.1 bis +53.5 
 */
void GMessage::setVoltage2(float volt){
  uint16_t val = (uint16_t) (volt * 10);
  uint8_t low =  (uint8_t) (val&255);
  uint8_t high = (uint8_t)(val >> 8);
  message_array[12] = low; 
  message_array[13] = high;    
}
/* 
 Einzelzellspannung:  +0.1 bis +5.08 Volt 
 */
void GMessage::setLipo1(float zelle){
  message_array[4] = (int) 100*zelle/2;   
}
/* 
 Einzelzellspannung:  +0.1 bis +5.08 Volt 
 */
void GMessage::setLipo2(float zelle){
  message_array[5] = (int) 100*zelle/2;   
}
/*
Einzelzellspannung:  +0.1 bis +5.08 Volt 
 */
void GMessage::setLipo3(float zelle){
  message_array[6] = (int) 100*zelle/2;   
}
/* 
 Einzelzellspannung:  +0.1 bis +5.08 Volt 
 */
void GMessage::setLipo4(float zelle){
  message_array[7] = (int) 100*zelle/2;   
}
/* 
 Einzelzellspannung:  +0.1 bis +5.08 Volt 
 */
void GMessage::setLipo5(float zelle){
  message_array[8] = (int) 100*zelle/2;   
}
/* 
 Einzelzellspannung:  +0.1 bis +5.08 Volt 
 */
void GMessage::setLipo6(float zelle){
  message_array[9] = (int) 100*zelle/2;   
} 
/* 
 Ampere: Werte +0.1 bis +53.5 
 */
void GMessage::setAmpere(float amp){
  uint16_t val = (uint16_t) (amp * 10);
  uint8_t low =  (uint8_t) (val&255);
  //Alternativ: uint8_t high = (uint8_t) ((val&65280)&256);
  uint8_t high = (uint8_t)(val >> 8);
  message_array[26] = low; 
  message_array[27] = high;    
} 
/* 
 milliAmpere: Werte +10 bis +65535 
 */
void GMessage::setMilliAmpere(uint32_t mA){
  mA = mA/10;
  uint8_t low =  (uint8_t) (mA&255);
  //Alternativ: uint8_t high = (uint8_t) ((val&65280)&256);
  uint8_t high = (uint8_t)(mA >> 8);
  message_array[30] = low; 
  message_array[31] = high;    
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/* 
 HÃƒÆ’Ã‚Â¶he in Metern (relativ): -500 bis +9999 [m], sinnvolle Werte z.B. 300 
 */
void GMessage::setAltitudeInMeters(uint16_t hoehe){
  uint8_t  hunderter = (int) (hoehe+500)/256;
  uint8_t  einer = hoehe-(hunderter*256-500);   
  message_array[22] = hunderter;
  message_array[21] = einer;
}

/* 
 setSteigrate(...) in Metern pro Sekunde: -296.8 bis +327.5 
 Die Implementation wirkt auf den ersten Blick etwas umstÃƒÆ’Ã‚Â¤ndlich.
 Ich habe diesen Weg gewählt, weil Arduino hier offenbar Rundungsprobleme 
 in einer Nachkommastelle hatte. Es ist nun zwar umstÃƒÆ’Ã‚Â¤ndlich, aber sicher 
 und transparent. Man sollte hier einen eleganteren 'workaround' schaffen.
 */
void GMessage::setClimbrate(float sr){
  //check validity first....
  if(sr < -296.9) sr = -296.9;
  if(sr > 327.5)  sr = 327.5;
  //calc msb Byte as refernce to "117" (ca. 0 Meter) because it start not with 0.... 
  uint8_t OFFSET_STEIGRATE = 117;
  int steigrate_meter = sr / 2.560+OFFSET_STEIGRATE;
  //calc the rest to compute the lsb    
  float rest = sr-(2.560*(steigrate_meter-OFFSET_STEIGRATE));
  //reference is 40, because Byte24=117 + Byte23=40 = 0.0 Meter)
  int OFFSET_DEZIMETER = 40;
  if(sr > 0) OFFSET_DEZIMETER = 50;
  int steigrate_dezimeter = OFFSET_DEZIMETER+rest*100;
  //check and prevent tooo big values.....
  if(steigrate_dezimeter > 0xff){
    steigrate_dezimeter = steigrate_dezimeter -0xff;
    steigrate_meter = steigrate_meter+1;
  }
  //Die entsprechende Gegenbedingung tritt offenbar nicht auf.... 
  //also steigrate_dezimeter < 0x00...
  //40 = 0 // 10 Ganzzahlschritte = 0.1 MS (0=-0.40, 10=-0.30 20=-0.2  30=-0.1 40=-0.0 ...]
  message_array[23] = steigrate_dezimeter; 
  //117 = 0 //  1 Ganzzahlschritt = 2.5600 M/S    [1= -297.0   10=-274.00  20=-248.40 ...]
  message_array[24] = steigrate_meter;  
}

/* 
 "Steigrate in 3 Sekunden": -120 bis + 135
 */
void GMessage::setClimbrate_M3(int srmmm){
  if(srmmm < -120) 
    srmmm = -120;
  if(srmmm > 135) 
    srmmm = 135;

  int steigrate = 120 +(int) srmmm;
  message_array[25] = steigrate;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* 
 Temperatursensor 1: -20 bis plus 235° C 
 */
void GMessage::setTemp1(int temp){
  if(temp < -20)
    temp = 20;
  if(temp > 234)
    temp = 235;
  message_array[14] = temp+20;
  ;   
}
/* 
 Temperatursensor 2: -20 bis plus 235Ãƒâ€šÃ‚Â° C 
 */
void GMessage::setTemp2(int temp){
  if(temp < -20)
    temp = 20;
  if(temp > 234)
    temp = 235;
  message_array[15] = temp+20;
}
/* 
 graphische Tankanzeige: 0%, 25%, 50%, 75%, 100%  
 */
void GMessage::setFuelPercent(uint8_t prozent){
  if(prozent > 100)
    prozent = 100;
  message_array[16] =(int) prozent-1;
}
/* 
 Tankfüllung: 0 bis 65535 ml, sinnvolle Milliliter-Werte z.B. 500 
 */
void GMessage::setFuelMilliliters(uint16_t ml){
  uint8_t  hunderter = (int) ml/256;
  uint8_t  einer = ml-(hunderter*256);   
  message_array[18] = hunderter;
  message_array[17] = einer;
}
/*
Umdrehungen pro Minute (PRM absolut): 0 bis 655350 [/min], sinnvolle RPM-Werte z.B. 12600 
 */
void GMessage::setRPM(uint32_t rpm){
  uint8_t tausender = (int) (rpm/2560);
  message_array[20] = tausender;
  uint8_t  zehner = (int) (rpm-(tausender*2560))/10;   
  message_array[19] = zehner;
}


void GMessage::checkLowVoltage(float voltage){
 
  //prevent that "ghost"-values are displayed e.g. after disconnecting
  setLipo1(0);
 
  //  message.accoustic_ALARM_OFF();
  if(voltage > (5*MIN_VOLT_1_CELL_LIPO) && voltage < (4*MAX_VOLT_1_CELL_LIPO))
  {
    //über 17.0 und unter 21 Volt
    accoustic_ALARM_OFF();
  }
  else if(voltage > (4*MIN_VOLT_1_CELL_LIPO) && voltage < (4*MAX_VOLT_1_CELL_LIPO))
  {
    //über 13.2 und unter 16.8 21 Volt
    accoustic_ALARM_OFF();
  }
  else if(voltage > (3*MIN_VOLT_1_CELL_LIPO) && voltage < (3*MAX_VOLT_1_CELL_LIPO))
  {
    //über 10.2 und unter 12.6 21 Volt
    accoustic_ALARM_OFF();
  }
  else if(voltage > (2*MIN_VOLT_1_CELL_LIPO) && voltage < (2*MAX_VOLT_1_CELL_LIPO))
  {
    //über 6.8 und unter 8.4 21 Volt
    accoustic_ALARM_OFF();
  }
  else if(voltage > (MIN_VOLT_1_CELL_LIPO) && voltage < (MAX_VOLT_1_CELL_LIPO))
  {
    //über 3.4 und unter 4.2
    accoustic_ALARM_OFF();
    setLipo1(voltage);
  }
  else {
    GMessage::accoustic_ALARM_LOW_VOLTAGE();
    
  }

  if(voltage < 1.0)
  {
    //if no measurement is connected
    GMessage::accoustic_ALARM_OFF();
  }
 
}



